#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   (C) Copyright 2009, APT-Portal Developers
#    https://launchpad.net/~apt-portal-devs
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This is a cherrypy / mako integration module


import cherrypy
import re
from urlparse import urlparse
from mako.template import Template
from mako.lookup import TemplateLookup
mylookup = None


""" 
	Sets the following directories:
		templates_directory - list of dirs to be searched for templates
		module_directory - directory to keep cached templates
"""
def set_directories(templates_directory, module_directory):
	global mylookup
		
	# Template rendering with internationalization support
	mylookup = TemplateLookup( \
		directories=templates_directory \
		, module_directory=module_directory \
		, input_encoding='utf-8' \
		, output_encoding='utf-8' \
		, encoding_errors='replace' \
		, default_filters=['stripnone'] \
		, imports=['from mako_filters import stripnone, html_lines']
	)

""" TODO: Translation using gettext """
def _(txt):
	return txt
	
def serve_template(templatename, **kwargs):
	global mylookup
	mytemplate = mylookup.get_template(templatename)

	# Global functions
	kwargs["_"] = _
	kwargs["session"] = session 
		
	# Check if we need to use a lang prefix
	kwargs["pagename"] = pagename()
	kwargs["base_url"] = cherrypy.request.base
	kwargs["self_url"] = cherrypy.request.path_info
	kwargs["release"] = cherrypy.request.release
	kwargs["login_username"] = None
	if cherrypy.session.has_key('login_username'):
		kwargs["login_username"] = cherrypy.session['login_username']
	return mytemplate.render(**kwargs)

def get_template_def(templatename, defname):
	global mylookup
	mytemplate = mylookup.get_template(templatename)
	return mytemplate.get_def(defname).render()
		
def validateEmail(email):

	if len(email) > 7:
		if re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", email) != None:
			return True
	return False

def validateUsername(username):

	if len(username) > 0:
		if re.match("^[a-zA-Z0-9\.]*$", username) != None:
			return True
	return False

"""
   The following are global variables extending the mako templates
"""
def pagename():
	"""
	"""
	page_parts = cherrypy.request.path_info.strip("/").split("/")
	if not page_parts:
		return None
	pagename = page_parts[0]
	#if pagename= "" and len(path_parts)>1:
	#    pagename = path_parts[len(path_parts) - 2]
	return pagename


"""
   The following are global functions  extending the mako templates
"""	
def session(key):
	""" Return value for a given session key, None if not found """
	if cherrypy.session.has_key(key):
		return cherrypy.session[key]
	return None

